//
// Created by jie on 2023/10/11.
//

#include "shaderService.h"
#include <fstream>
#include <iostream>

ShaderService::ShaderService(const std::filesystem::path &vertexShaderPath,
                             const std::filesystem::path &fragShaderPath) {
    if(!(exists(vertexShaderPath) && exists(fragShaderPath))){
        return;
    }

    std::stringstream ss, sss;
    std::ifstream verIn{};
    std::ifstream fragIn{};
    verIn.open(vertexShaderPath);
    fragIn.open(fragShaderPath);
    ss << verIn.rdbuf();
    const auto vertexSource = ss.str();
    sss << fragIn.rdbuf();
    const auto fragmentSource = sss.str();
    verIn.close();fragIn.close();

    const char* vSource = vertexSource.c_str();
    const char* fSource = fragmentSource.c_str();

    unsigned int frag, vertex;
    frag = glCreateShader(GL_FRAGMENT_SHADER);
    vertex = glCreateShader(GL_VERTEX_SHADER);
    glShaderSource(vertex, 1, &vSource, nullptr);
    glShaderSource(frag, 1, &fSource, nullptr);
    glCompileShader(vertex);
    glCompileShader(frag);
    if(const auto res = (CheckShader(vertex) && CheckShader(frag));!res){
        return;
    }
    programId = glCreateProgram();
    glAttachShader(programId, vertex);
    glAttachShader(programId, frag);
    glLinkProgram(programId);
    CheckShader(programId, true);

    glDeleteShader(vertex);
    glDeleteShader(frag);
}

void ShaderService::Use() {
    glUseProgram(programId);
}

bool  ShaderService::CheckShader(const unsigned int shaderIndex, bool isProgram) {
    int successful;
    char logInfo[512];
    if (isProgram) {
        glad_glGetProgramiv(shaderIndex, GL_LINK_STATUS, &successful);
    } else {
        glad_glGetShaderiv(shaderIndex, GL_COMPILE_STATUS, &successful);
    }
    if (!successful) {
        if (isProgram) {
            glad_glGetProgramInfoLog(shaderIndex, 512, nullptr, logInfo);
            std::cout<<logInfo<<std::endl;
        } else {
            glad_glGetShaderInfoLog(shaderIndex, 512, nullptr, logInfo);
            std::cout<<logInfo<<std::endl;
        }
        return false;
    }
    return true;
}