#include "triangle.h"
#include "iostream"
#include <chrono>
using namespace std::literals::chrono_literals;
namespace TRIANGLE {

    void frameBufferSizeChange(GLFWwindow *window, int width, int height) {
        glViewport(0, 0, width, height);
    }

    void processInput(GLFWwindow *window) {
        if (glfwGetKey(window, GLFW_KEY_ESCAPE) == GLFW_PRESS) {
            glfwSetWindowShouldClose(window, true);
        }
    }

    void CheckShader(unsigned int shaderIndex, bool isPorgram) {
        int successful;
        char logInfo[512];
        if (isPorgram) {
            glad_glGetProgramiv(shaderIndex, GL_LINK_STATUS, &successful);
        } else {
            glad_glGetShaderiv(shaderIndex, GL_COMPILE_STATUS, &successful);
        }
        if (!successful) {
            if (isPorgram) {
                glad_glGetProgramInfoLog(shaderIndex, 512, nullptr, logInfo);
            } else {
                glad_glGetShaderInfoLog(shaderIndex, 512, nullptr, logInfo);
            }
            std::cout << logInfo << std::endl;
        }
    }

    int Practice1() {
        glfwInit();
        glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
        glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
        glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

        GLFWwindow *window = glfwCreateWindow(800, 600, "Windows", nullptr, nullptr);
        if (window == nullptr) {
            return -1;
        }
        glfwMakeContextCurrent(window);
        glfwSetWindowSizeCallback(window, frameBufferSizeChange);

        if (!gladLoadGLLoader((GLADloadproc) glfwGetProcAddress)) {
            auto error = glad_glGetError();
            return static_cast<int>(error);
        }

        float vertices[] = {
                0.5f, -0.5f, 0.0f, 1.0f, 0.0f, 0.0f,
                -0.5f, -0.5f, 0.0f, 0.0f, 1.0f, 0.0f,
                0.0f, 0.5f, 0.0f, 0.0f, 0.0f, 1.0f
        };

        unsigned int VAO, VBO, EBO;
        glGenVertexArrays(1, &VAO);
        glGenBuffers(1, &VBO);
        glGenBuffers(1, &EBO);

        glBindVertexArray(VAO);
        glBindBuffer(GL_ARRAY_BUFFER, VBO);

        glBufferData(GL_ARRAY_BUFFER, sizeof(vertices), vertices, GL_STATIC_DRAW);
        glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 6 * sizeof(float), (void *) 0);
        glEnableVertexAttribArray(0);
        glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, 6*sizeof(float), (void*)(3*sizeof(float)));
        glEnableVertexAttribArray(1);
        glBindVertexArray(0);


        auto shader = ShaderService("../../shaders/vertexShader.glsl",
                                          "../../shaders/fragmentShader.glsl");

        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
        float pos = .0f;
        while (!glfwWindowShouldClose(window)) {
            processInput(window);

            glClearColor(.2f, .3f, .3f, 1.0f);
            glClear(GL_COLOR_BUFFER_BIT);

            shader.Use();
            shader.SetUniform<float>("position", pos+=0.001);
//            std::this_thread::sleep_for(5ms);

            glBindVertexArray(VAO);
//            glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_INT, 0);
//            glUseProgram(shaderProgram[1]);
            glDrawArrays(GL_TRIANGLES, 0, 3);
            glfwSwapBuffers(window);
            glfwPollEvents();
        }

        glDeleteBuffers(1, &VBO);
        glDeleteVertexArrays(1, &VAO);
        glDeleteBuffers(1, &EBO);

        glfwTerminate();
        return 0;
    }
}